import json
from pathlib import Path
from jsonschema import ValidationError, RefResolver
from jsonschema.validators import Draft202012Validator


class Validator:
    def __init__(self, json_schema: Path, json_suffix: str = "*.json") -> None:
        self.json_schema = json_schema
        self.json_suffix = json_suffix
        print(f"Validator for schema: {json_schema} \nSuffix: {json_suffix}\n")

    def file(self, file_path: Path) -> bool:
        with open(str(self.json_schema)) as schema_file, open(
            str(file_path)
        ) as data_file:
            schema = json.load(schema_file)
            data = json.load(data_file)
            schema_path = self.json_schema.parent.absolute().as_uri()
            resolver = RefResolver(base_uri=schema_path, referrer=schema)

            try:
                Draft202012Validator(
                    schema=schema,
                    format_checker=Draft202012Validator.FORMAT_CHECKER,
                    resolver=resolver,
                ).validate(data)
                print(f"Validate file: {file_path.parent} / {file_path.name}\n")

            except ValidationError as e:
                print("Validation error:", e)

        return True

    def folder(self, folder_path: Path) -> bool:
        print(f"Validate folder: {folder_path}\n")
        for file in folder_path.glob(self.json_suffix):
            self.file(file)
        return True

    def get_schema(self):
        with open(str(self.json_schema)) as schema_file:
            schema = json.load(schema_file)
            schema_path = self.json_schema.parent.absolute().as_uri()
            resolver = RefResolver(base_uri=schema_path, referrer=schema)

            return self.resolve_ref(resolver, schema)

    def resolve_ref(self, resolver, schema_part):
        if isinstance(schema_part, dict) and "$ref" in schema_part:
            # Resolve the reference
            ref = schema_part["$ref"]
            with resolver.resolving(ref) as resolved:
                return self.resolve_ref(
                    resolver, resolved
                )  # Recursively resolve nested references
        elif isinstance(schema_part, dict):
            # Resolve each value in the dictionary
            return {
                key: self.resolve_ref(resolver, value)
                for key, value in schema_part.items()
            }
        elif isinstance(schema_part, list):
            # Resolve each item in the list
            return [self.resolve_ref(resolver, item) for item in schema_part]
        return schema_part
